package sun.misc;

import java.util.Comparator;

public class ASCIICaseInsensitiveComparator implements Comparator<String>
{
    public static final Comparator<String> CASE_INSENSITIVE_ORDER;
    
    public ASCIICaseInsensitiveComparator() {
    }
    
    @Override
    public int compare(String s, String s2) {
        int length = s.length();
        int length2 = s2.length();
        length2 = ((length < length2) ? length : length2);
        int i = 0;
        while (i < length2) {
            int char1 = s.charAt(i);
            int char2 = s2.charAt(i);
            assert char1 <= '\u007f' && char2 <= '\u007f';
            if (char1 != char2) {
                int n = (char)toLower(char1);
                int n2 = (char)toLower(char2);
                if (n != n2) {
                    return n - n2;
                }
            }
            ++i;
        }
        return length - length2;
    }
    
    public static int lowerCaseHashCode(String s) {
        int n = 0;
        for (int length = s.length(), i = 0; i < length; ++i) {
            n = 31 * n + toLower((int)s.charAt(i));
        }
        return n;
    }
    
    static boolean isLower(int n) {
        return (n - 97 | 122 - n) >= 0;
    }
    
    static boolean isUpper(int n) {
        return (n - 65 | 90 - n) >= 0;
    }
    
    static int toLower(int n) {
        return isUpper(n) ? (n + 32) : n;
    }
    
    static int toUpper(int n) {
        return isLower(n) ? (n - 32) : n;
    }
    
    static {
        CASE_INSENSITIVE_ORDER = new ASCIICaseInsensitiveComparator();
    }
}
